﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Diagrams;
using Telerik.Windows.Diagrams.Core;

namespace DiagramSL
{
    public class CustomDiagram : RadDiagram
    {
        private DiagramSurface secondSurface;
        private DiagramSurface originalSurface;

        //private List<IShape> MyShapes
        //{
        //    get
        //    {
        //        return this.Shapes. as List<IShape>;
        //    }
        //}

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            this.secondSurface = this.GetTemplateChild("secondSurface") as DiagramSurface;
            this.originalSurface = this.GetTemplateChild("ItemsHost") as DiagramSurface;

            List<RadDiagramShape> noZoomShapes = new List<RadDiagramShape>();
            if (originalSurface != null)
            {
                foreach (var item in this.Shapes)
                {
                    var shape = item as RadDiagramShape;
                    if (shape == null || !"NoZoom".Equals(shape.Tag))
                        continue;

                    this.MoveItemToSecondHost(shape);
                    //noZoomShapes.Add(shape);
                    //if (originalSurface.Children.Contains(shape))
                    //    originalSurface.Children.Remove(shape);
                }
            }

            //if (this.secondSurface != null)
            //{
            //    this.secondSurface.Children.Clear();
            //    foreach (var shape in noZoomShapes)
            //    {
            //        this.secondSurface.Children.Add(shape);
            //    }
            //}
        }

        protected override bool PublishDiagramEvent(Telerik.Windows.Diagrams.Core.DiagramEvent diagramEvent, object args)
        {
            var result = base.PublishDiagramEvent(diagramEvent, args);
            if (diagramEvent == DiagramEvent.ShapeAdded)
            {

                var shapeArgs = args as GenericEventArgs<IShape>;
                var shape = shapeArgs.Entity as RadDiagramShape;
                if ("NoZoom".Equals(shape.Tag))
                {
                    this.MoveItemToSecondHost(shape);
                }


                var element = shapeArgs.Entity as FrameworkElement;
                if (element == null) return false;

                if (this.secondSurface != null && this.secondSurface.Children.Contains(element))
                {
                    this.secondSurface.Children.Add(element);
                }
            }
            else if (diagramEvent == DiagramEvent.ShapeRemoved)
            {
                var shapeArgs = args as GenericEventArgs<IShape>;
                var shape = shapeArgs.Entity as RadDiagramShape;
                if ("NoZoom".Equals(shape.Tag) && this.secondSurface != null)
                {
                    this.secondSurface.Children.Remove(shape);
                }
            }
            else if (diagramEvent == DiagramEvent.PanDelta)
            {
                var positionChangedEventArgs = args as PositionChangedEventArgs;

                if (this.secondSurface != null)
                {
                    var transformation = this.secondSurface.RenderTransform as TranslateTransform;
                    transformation.X = positionChangedEventArgs.NewPosition.X;
                    transformation.Y = positionChangedEventArgs.NewPosition.Y;
                }

                //this.PanInternal(positionChangedEventArgs.NewPosition, useAnimation);
            }

            return result;
        }

        private void MoveItemToSecondHost(RadDiagramShape shape)
        {
            if (this.originalSurface != null && this.originalSurface.Children.Contains(shape))
            {
                this.originalSurface.Children.Remove(shape);
            }

            if (this.secondSurface != null && !this.secondSurface.Children.Contains(shape))
            {
                this.secondSurface.Children.Add(shape);
            }
        }
    }
}
